{
  Promathika - ein multifunktionales Programm zum Zeichnen von Funktionen

    Copyright (C) 2009  Arthur Schmidt, Alexander Staidl

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 3 as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
}


unit u_prm_settings;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, Dialogs,FileUtil;


  type tColorSettings=record
    bgrdCol:tColor;       //ID 0000
    bgrdCol2:tColor;      //ID 0001
    CurvCol:tColor;       //ID 0002
    CurvCol2:tColor;      //ID 0003
    GridCol:tColor;       //ID 0004
    PinCol: tColor;       //ID 0005
    AxisCol:tColor;       //ID 0006
    AxisCrossCol:tColor;  //ID 0007
    RulBgrdCol:tColor;    //ID 0008
    RulTextCol:tColor;    //ID 0009
    BgrdGrad:ShortInt;    //ID 0010  Hintergrundfarbverlauf  -1=aus; 0=vertical; 1=horizontal
    CurvGrad:ShortInt;    //ID 0011  Funktionsfarbverlauf  -1=aus; 0=vertical; 1=horizontal
  end;

  type tPerformanceSet=record
    ThreadCount:byte;          //ID 1000  Anzahl der Threads für Graphberechnung
    CurvWidth:double;          //ID 1001  Strichbreite für Graphen
    AA_Mode:word;              //ID 1002  1=Tetragon; 2=Hexagon; 3=Kreis; 4=Rundeck
    AA_Level_hor:byte;         //ID 1003  horizontale Subpixel-Anzahl
    AA_Level_ver:byte;         //ID 1004  vertikale Subpixelanzahl
    unsharpnes:double;         //ID 1005  Schritweite in Hauptachsen-Richtung
    TermOut:ShortInt;          //ID 1006  -1=aus; 0=nur Term; 1=Nur Name; 2=Name & Term
    TermkoordX:integer;        //ID 1007  X-Koordinate für linke obere Ausgabe-Ecke von Termen in %
    TermKoordY:integer;        //ID 1008  Y-Koordinate in %
    statistics:boolean;        //ID 1009  Auswertungen der Rechenzeit-Verteilung zulassen
  end;

  type tKartesianSet=record
    xmin:double;            //ID 2000  Definition des darzustellenden mathematischen Bereichs
    xmax:double;            //ID 2001  Definition des darzustellenden mathematischen Bereichs
    ymin:double;            //ID 2002  Definition des darzustellenden mathematischen Bereichs
    ymax:double;            //ID 2003  Definition des darzustellenden mathematischen Bereichs
    DistortFree:ShortInt;   //ID 2004  -1=aus;0=xmin variabel; 1=xmax variabel; 2=ymin variabel; 3=ymax variabel
    AxisSign:boolean;       //ID 2005  Achsenkreuz-Symbol
    Ruldens_X:double;       //ID 2006  Beschriftungsdichte Lineal in X
    Ruldens_Y:double;       //ID 2007  Beschriftungsdichte Lineal in Y
    GridDensX:double;       //ID 2008  Gitterdichte in X-Richtung
    GridDensY:double;       //ID 2009  Gitterdichte in Y-Richtung
    PinSubDensX:double;     //ID 2010  Zusatzunterteilungsfaktor in X-Richtung für Skalenstriche  (für Achsen und Lineal gleichermaßen gültig)
    PinSubDensY:double;     //ID 2011  Zusatzunterteilungsfaktor in Y-Richtung für Skalenstriche  (für Achsen und Lineal gleichermaßen gültig)
    RulerMode:ShortInt;     //ID 2012  -1=aus; 0=angepasst; 1=benutzerdefinierter Hintergrund
    dynamicGrid:boolean;    //ID 2013  erlaubt dynamische Anpassungen der Gitterdichte an sinnvolle Werte
    UseUsrDefUnits:boolean; //ID 2014  Nutzung von Benutzerdefinierten Einheiten
    UnitValueX:double;      //ID 2015  Wertigkeit der x-Einheit in x-Richtung
    UnitValueY:double;      //ID 2016  Wertigkeit der y-Einheit in y-Richtung
    UnitSignX:String[5];    //ID 2017  Einheitskürzel in x-Richtung, z.B.: Pi, N, MPa, Km...
    UnitSignY:String[5];    //ID 2018  Einheitskürzel in y-Richtung, z.B.: Pi, N, MPa, Km...
    UnitNameX:String[16];   //ID 2019  Einheitsbezeichnung in x-Richtung, z.B.: Kraft, Weg, Energie...
    UnitNameY:String[16];   //ID 2020  Einheitsbezeichnung in y-Richtung, z.B.: Kraft, Weg, Energie...
    DrwUSignRul:boolean;    //ID 2021  Entscheidet, ob Einheitskürzel im Lineal mitgezeichnet wird, wenn benutzerdefiniertes System aktiv
  end;

  type tMainSizing=record
    DrawResX:word;          //ID 3000
    DrawResY:word;          //ID 3001
    MainWidth:word;         //ID 3002
    MainHeight:word;        //ID 3003
    Maximized:boolean;      //ID 3004
  end;

  type tFontSet=record
    StatsFontScale:single;  //ID 4000  Skalierung der automatischen Schriftgrößenanpassung für Statistikausgaben
    RulFontScale:single;    //ID 4001  Skalierung der automatischen Schriftgrößenanpassung für Linealausgaben
    AxisFontScale:single;   //ID 4002  Skalierung der automatischen Schriftgrößenanpassung für Achsenkreuz-Symbol
  end;

  type tGlobalSet=record
    ColorSet:tColorSettings;
    PerformanceSet:tPerformanceSet;
    Kartesianset:tKartesianSet;
    MainSizing:tMainSizing;
    FontSet:tFontSet;
  end;

  type tStrArr=Array of String;

  type tPrmSettings =class
    private
      LastfuncsPath,IniPath:UTF8string;
      SettingsArr:Array [0..1]of Array of tGlobalSet;//0=MainSource; 1=ListSource
      LastMainEditFuncs:tStrArr;
      procedure StrToOption(OptionLine:string; var workSet:tGlobalSet); //===>> Neue Settings müssen hier registriert werden für das Laden
      function SearchLast(const SearchStr:String):longInt;    //Wenn nichts gefunden wird=>false, sonst true
    public
      constructor create;
      function get_Defaults:tGlobalSet;
      function change_viewRange(set_input:tGlobalSet):tGlobalSet;
      function SaveGlobalSetToIni(const ThisSet:tGlobalSet; NameOrPath:UTF8string):boolean; //===>> Neue Settings müssen hier registriert werden zum Speichern
      function LoadIniToGlobalSet(var RefSet:tGlobalSet; NameOrPath:UTF8string):boolean;
      function UndoSettings (MainSource:boolean;const OldSettings:tGlobalSet):tGlobalSet;
      procedure BackupSettings (MainSource:boolean;const Settings:tGlobalSet);
      procedure DelSettingsBackUps (MainSource:boolean);
      function BackUpCount(const MainSource:boolean):word;
      function LoadLastFuncs (var LoadAim:tStrArr; loc_filename:String):boolean;
      function AddNewLastFunc (thisfunction:String):boolean; //Wenn neue Funktion hinzugefügt worden ist, dann =true, sonst false
      function SaveLastFuncs (loc_filename:String):boolean;
      procedure refreshLastFuncs (var ref_results:tStrArr);
      destructor free;
    end;

implementation
  constructor tPrmSettings.create;
    begin
      Inherited Create;
      Setlength(SettingsArr[0],0);
      Setlength(SettingsArr[1],0);
      Setlength(LastMainEditFuncs,0);
      LastfuncsPath:='';
      IniPath:='';
    end;

  Destructor tPrmSettings.free;
    begin
      SettingsArr[0]:=nil;
      SettingsArr[1]:=nil;
      LastMainEditFuncs:=nil;
      inherited Destroy;
    end;

  function tprmSettings.get_Defaults:tGlobalSet;
    begin
      with result.ColorSet do begin
        bgrdCol:=clwhite;
        bgrdCol2:=clsilver;
        CurvCol:=Clblue;
        CurvCol2:=Clblue;
        GridCol:=clSilver;
        PinCol:=clRed;
        AxisCol:=clBlack;
        AxisCrossCol:=clblack;
        RulBgrdCol:=clwhite;
        RulTextCol:=clnavy;
        BgrdGrad:=-1;
        CurvGrad:=-1;
      end;

      with result.PerformanceSet do begin
        ThreadCount:=4;
        CurvWidth:=1.5;
        AA_Mode:=2;
        AA_Level_hor:=4;
        AA_Level_ver:=4;
        unsharpnes:=0.1078;
        TermOut:=-1;
        TermkoordX:=5;
        TermkoordY:=5;
        statistics:=false;
      end;

      with result.KartesianSet do begin
        xmin:=-9;
        xmax:=9;
        ymin:=-5;
        ymax:=5;
        DistortFree:=-1;//-1=aus;0=xmin variabel; 1=xmax variabel; 2=ymin variabel; 3=ymax variabel
        AxisSign:=true;
        Ruldens_X:=1;
        Ruldens_Y:=1;
        GridDensX:=1;
        GridDensY:=1;
        PinSubDensX:=1;
        PinSubDensY:=1;
        RulerMode:=0;
        dynamicGrid:=true;
        UseUsrDefUnits:=false; //ID 2015  Nutzung von Benutzerdefinierten Einheiten
        UnitValueX:=1;         //ID 2016  Wertigkeit der x-Einheit in x-Richtung
        UnitValueY:=1;         //ID 2017  Wertigkeit der y-Einheit in y-Richtung
        UnitSignX:='';         //ID 2018  Einheitskürzel in x-Richtung, z.B.: Pi, N, MPa, Km...
        UnitSignY:='';         //ID 2019  Einheitskürzel in y-Richtung, z.B.: Pi, N, MPa, Km...
        UnitNameX:='';         //ID 2020  Einheitsbezeichnung in x-Richtung, z.B.: Kraft, Weg, Energie...
        UnitNameY:='';         //ID 2021  Einheitsbezeichnung in y-Richtung, z.B.: Kraft, Weg, Energie...
        DrwUSignRul:=true;     //ID 2022  Entscheidet, ob das Einheitskürzel mit abgebildet wird im Lineal, falls Benutzerdefiniertes Einheitensystem
      end;

      with result.MainSizing do begin
        MainWidth:=900;
        MainHeight:=520;
        DrawResX:=900-48;
        DrawResY:=520-140;
        Maximized:=false;
      end;

      with result.FontSet do begin
        StatsFontScale:=1.0;
        RulFontScale:=1.0;
        AxisFontScale:=1.0;
      end;
    end;

  function tprmsettings.change_viewRange(set_input:tGlobalSet):tGlobalSet;
    begin
      if (set_input.Kartesianset.xmin<set_input.Kartesianset.xmax)and(set_input.Kartesianset.ymin<set_input.Kartesianset.ymax) then begin

        result:=set_input;

        if Set_input.Kartesianset.DistortFree=0 then begin  //xmin ist variabel
          result.Kartesianset.xmin:=result.Kartesianset.xmax-(result.mainsizing.DrawResX/(result.mainsizing.DrawResY/(result.Kartesianset.ymax-result.Kartesianset.ymin)));
        end;

        if Set_input.Kartesianset.DistortFree=1 then begin  //xmax ist variabel
          result.Kartesianset.xmax:=result.Kartesianset.xmin+(result.mainsizing.DrawResX/(result.mainsizing.DrawResY/(result.Kartesianset.ymax-result.Kartesianset.ymin)));
        end;

        if Set_input.Kartesianset.DistortFree=2 then begin  //ymin ist variabel
          result.Kartesianset.ymin:=result.Kartesianset.ymax-(result.mainsizing.DrawResY/(result.mainsizing.DrawResX/(result.Kartesianset.xmax-result.Kartesianset.xmin)));
        end;

        if Set_input.Kartesianset.DistortFree=3 then begin  //ymax ist variabel
          result.Kartesianset.ymax:=result.Kartesianset.ymin+(result.mainsizing.DrawResY/(result.mainsizing.DrawResX/(result.Kartesianset.xmax-result.Kartesianset.xmin)));
        end;

      end else MessageDlg('*Eingabefehler*','Es wird nichts geändert, da mindestens ein Minimalwert größer ist als der entsprechende Maximalwert', mtError, [mbOk] , 0);
    end;

  function tprmsettings.SaveGlobalSetToIni(const ThisSet:tGlobalSet; NameOrPath:UTF8string):boolean;
    var
      DefaultSet:TextFile;
   begin
     result:=false;
     If length(NameOrPath)>0 then begin
     try
       AssignFile(DefaultSet,UTF8ToSys(NameOrPath));
       {$i-}
       ReWrite(DefaultSet);
       {$i+}

       writeln(DefaultSet,'ColorSet =================');
       writeln(DefaultSet,'0000 #'+'bgrdCol| '+IntToStr(ThisSet.ColorSet.bgrdCol));
       writeln(DefaultSet,'0001 #'+'bgrdCol2| '+IntToStr(ThisSet.ColorSet.bgrdCol2));
       writeln(DefaultSet,'0002 #'+'CurvCol| '+IntToStr(ThisSet.ColorSet.CurvCol));
       writeln(DefaultSet,'0003 #'+'CurvCol2| '+IntToStr(ThisSet.ColorSet.CurvCol2));
       writeln(DefaultSet,'0004 #'+'GridCol| '+IntToStr(ThisSet.ColorSet.GridCol));
       writeln(DefaultSet,'0005 #'+'PinCol| '+IntToStr(ThisSet.ColorSet.PinCol));
       writeln(DefaultSet,'0006 #'+'AxisCol| '+IntToStr(ThisSet.ColorSet.AxisCol));
       writeln(DefaultSet,'0007 #'+'AxisCrossCol| '+IntToStr(ThisSet.ColorSet.AxisCrossCol));
       writeln(DefaultSet,'0008 #'+'RulBgrdCol| '+IntToStr(ThisSet.ColorSet.RulBgrdCol));
       writeln(DefaultSet,'0009 #'+'RulTextCol| '+IntToStr(ThisSet.ColorSet.RulTextCol));
       writeln(DefaultSet,'0010 #'+'BgrdGrad| '+IntToStr(ThisSet.ColorSet.BgrdGrad));
       writeln(DefaultSet,'0011 #'+'CurvGrad| '+IntToStr(ThisSet.ColorSet.CurvGrad));

       writeln(DefaultSet,''); writeln(DefaultSet,'PerformanceSet =================');
       writeln(DefaultSet,'1000 #'+'ThreadCount| '+IntToStr(ThisSet.PerformanceSet.ThreadCount));
       writeln(DefaultSet,'1001 #'+'CurvWidth| '+FloatToStr(ThisSet.PerformanceSet.CurvWidth));
       writeln(DefaultSet,'1002 #'+'AA_Mode| '+IntToStr(ThisSet.PerformanceSet.AA_Mode));
       writeln(DefaultSet,'1003 #'+'AA_Level_hor| '+IntToStr(ThisSet.PerformanceSet.AA_Level_hor));
       writeln(DefaultSet,'1004 #'+'AA_Level_ver| '+IntToStr(ThisSet.PerformanceSet.AA_Level_ver));
       writeln(DefaultSet,'1005 #'+'unsharpnes| '+FloatToStr(ThisSet.PerformanceSet.unsharpnes));
       writeln(DefaultSet,'1006 #'+'TermOut| '+IntToStr(ThisSet.PerformanceSet.TermOut));
       writeln(DefaultSet,'1007 #'+'TermKoordX| '+IntToStr(ThisSet.PerformanceSet.TermkoordX));
       writeln(DefaultSet,'1008 #'+'TermKoordY| '+IntToStr(ThisSet.PerformanceSet.TermkoordY));
       writeln(DefaultSet,'1009 #'+'statistics| '+BoolToStr(ThisSet.PerformanceSet.statistics));

       writeln(DefaultSet,''); writeln(DefaultSet,'KartesianSet =================');
       writeln(DefaultSet,'2000 #'+'xmin| '+FloatToStr(ThisSet.Kartesianset.xmin));
       writeln(DefaultSet,'2001 #'+'xmax| '+FloatToStr(ThisSet.Kartesianset.xmax));
       writeln(DefaultSet,'2002 #'+'ymin| '+FloatToStr(ThisSet.Kartesianset.ymin));
       writeln(DefaultSet,'2003 #'+'ymax| '+FloatToStr(ThisSet.Kartesianset.ymax));
       writeln(DefaultSet,'2004 #'+'DistortionFree| '+IntToStr(ThisSet.Kartesianset.DistortFree));
       writeln(DefaultSet,'2005 #'+'AxisSign| '+BoolToStr(ThisSet.Kartesianset.AxisSign));
       writeln(DefaultSet,'2006 #'+'RulDensX| '+FloatToStr(ThisSet.Kartesianset.Ruldens_X));
       writeln(DefaultSet,'2007 #'+'RulDensY| '+FloatToStr(ThisSet.Kartesianset.Ruldens_Y));
       writeln(DefaultSet,'2008 #'+'GridDensX| '+FloatToStr(ThisSet.Kartesianset.GridDensX));
       writeln(DefaultSet,'2009 #'+'GridDensY| '+FloatToStr(ThisSet.Kartesianset.GridDensY));
       writeln(DefaultSet,'2010 #'+'PinSubDensX| '+FloatToStr(ThisSet.Kartesianset.PinSubDensX));
       writeln(DefaultSet,'2011 #'+'PinSubDensY| '+FloatToStr(ThisSet.Kartesianset.PinSubDensY));
       writeln(DefaultSet,'2012 #'+'RulerMode| '+IntToStr(ThisSet.Kartesianset.RulerMode));
       writeln(DefaultSet,'2013 #'+'DynamicGrid| '+BoolToStr(ThisSet.Kartesianset.dynamicGrid));
       writeln(DefaultSet,'2014 #'+'UseUsrDefUnits| '+BoolToStr(ThisSet.Kartesianset.UseUsrDefUnits));
       writeln(DefaultSet,'2015 #'+'UnitValueX| '+FloatToStr(ThisSet.Kartesianset.UnitValueX));
       writeln(DefaultSet,'2016 #'+'UnitValueY| '+FloatToStr(ThisSet.Kartesianset.UnitValueY));
       writeln(DefaultSet,'2017 #'+'UnitSignX| '+ThisSet.Kartesianset.UnitSignX);
       writeln(DefaultSet,'2018 #'+'UnitSignY| '+ThisSet.Kartesianset.UnitSignY);
       writeln(DefaultSet,'2019 #'+'UnitNameX| '+ThisSet.Kartesianset.UnitNameX);
       writeln(DefaultSet,'2020 #'+'UnitNameY| '+ThisSet.Kartesianset.UnitNameY);
       writeln(DefaultSet,'2021 #'+'DrwUSingRul| '+BoolToStr(ThisSet.Kartesianset.DrwUSignRul));

       writeln(DefaultSet,''); writeln(DefaultSet,'MainSizing =================');
       writeln(DefaultSet,'3000 #'+'DrawResX| '+IntToStr(ThisSet.MainSizing.DrawResX));
       writeln(DefaultSet,'3001 #'+'DrawResY| '+IntToStr(ThisSet.MainSizing.DrawResY));
       writeln(DefaultSet,'3002 #'+'MainWidth| '+IntToStr(ThisSet.MainSizing.MainWidth));
       writeln(DefaultSet,'3003 #'+'MainHeight| '+IntToStr(ThisSet.MainSizing.MainHeight));
       writeln(DefaultSet,'3004 #'+'Maximized| '+BoolToStr(ThisSet.MainSizing.Maximized));

       writeln(DefaultSet,''); writeln(DefaultSet,'FontSet =================');
       writeln(DefaultSet,'4000 #'+'StatsFontScale| '+FloatToStr(ThisSet.FontSet.StatsFontScale));
       writeln(DefaultSet,'4001 #'+'RulFontScale| '+FloatToStr(ThisSet.FontSet.RulFontScale));
       writeln(DefaultSet,'4002 #'+'AxisFontScale| '+FloatToStr(ThisSet.FontSet.AxisFontScale));

       result:=true;
     finally
       CloseFile(DefaultSet);
     end;
     end;
    end;

  procedure tprmsettings.StrToOption(OptionLine:String; var workSet:tGlobalSet);
   var
     OptIDBuffer:longint=-1;
     PosBuf:array[0..2]of longint;
    begin
      If length(OptionLine)>0 then begin
        PosBuf[0]:=Pos('#',Optionline);
        PosBuf[1]:=Pos('|',OptionLine);
        PosBuf[2]:=Pos('@',OptionLine);

      end;
        If (PosBuf[0]>0)and(PosBuf[1]>PosBuf[0]+1)then begin
          OptIdBuffer:=StrToInt(trim(Copy(OptionLine,1,PosBuf[0]-1)));
          If PosBuf[2]>PosBuf[1]+1 then OptionLine:=trim(copy(OptionLine,PosBuf[1]+1,PosBuf[2]-PosBuf[1]-1)) else
           If PosBuf[2]=0 then OptionLine:=trim(Copy(OptionLine,Posbuf[1]+1,length(OptionLine)-PosBuf[1])) else OptIdBuffer:=-1;

          If (OptIDBuffer>-1) then begin
            Case OptIDBuffer of
            //ColorSet
            0: WorkSet.ColorSet.bgrdCol:=StrToInt(OptionLine);
            1: WorkSet.ColorSet.bgrdCol2:=StrToInt(OptionLine);
            2: WorkSet.ColorSet.CurvCol:=StrToInt(OptionLine);
            3: WorkSet.ColorSet.CurvCol2:=StrToInt(OptionLine);
            4: WorkSet.ColorSet.GridCol:=StrToInt(OptionLine);
            5: WorkSet.ColorSet.PinCol:=StrToInt(OptionLine);
            6: WorkSet.ColorSet.AxisCol:=StrToInt(OptionLine);
            7: WorkSet.ColorSet.AxisCrossCol:=StrToInt(OptionLine);
            8: WorkSet.ColorSet.RulbgrdCol:=StrToInt(OptionLine);
            9: WorkSet.ColorSet.RulTextCol:=StrToInt(OptionLine);
            10: WorkSet.ColorSet.bgrdGrad:=StrToInt(OptionLine);
            11: WorkSet.ColorSet.CurvGrad:=StrToInt(OptionLine);
            //PerformanceSet
            1000: WorkSet.PerformanceSet.ThreadCount:=StrToInt(OptionLine);
            1001: WorkSet.PerformanceSet.CurvWidth:=StrToFloat(OptionLine);
            1002: WorkSet.PerformanceSet.AA_Mode:=StrToInt(OptionLine);
            1003: WorkSet.PerformanceSet.AA_Level_hor:=StrToInt(OptionLine);
            1004: WorkSet.PerformanceSet.AA_Level_ver:=StrToInt(OptionLine);
            1005: WorkSet.PerformanceSet.unsharpnes:=StrToFloat(OptionLine);
            1006: WorkSet.PerformanceSet.TermOut:=StrToInt(OptionLine);
            1007: WorkSet.PerformanceSet.TermKoordX:=StrToInt(OptionLine);
            1008: WorkSet.PerformanceSet.TermKoordY:=StrToInt(OptionLine);
            1009: WorkSet.PerformanceSet.Statistics:=StrToBool(OptionLine);
            //KartesianSet
            2000: WorkSet.Kartesianset.xmin:=StrToFloat(OptionLine);
            2001: WorkSet.Kartesianset.xmax:=StrToFloat(OptionLine);
            2002: WorkSet.Kartesianset.ymin:=StrToFloat(OptionLine);
            2003: WorkSet.Kartesianset.ymax:=StrToFloat(OptionLine);
            2004: WorkSet.Kartesianset.DistortFree:=StrToInt(OptionLine);
            2005: WorkSet.Kartesianset.AxisSign:=StrToBool(OptionLine);
            2006: WorkSet.Kartesianset.Ruldens_X:=StrToFloat(OptionLine);
            2007: WorkSet.Kartesianset.Ruldens_Y:=StrToFloat(OptionLine);
            2008: WorkSet.Kartesianset.GridDensX:=StrToFloat(OptionLine);
            2009: WorkSet.Kartesianset.GridDensY:=StrToFloat(OptionLine);
            2010: WorkSet.Kartesianset.PinSubDensX:=StrToFloat(OptionLine);
            2011: WorkSet.Kartesianset.PinSubDensY:=StrToFloat(OptionLine);
            2012: WorkSet.Kartesianset.RulerMode:=StrToInt(OptionLine);
            2013: WorkSet.Kartesianset.dynamicGrid:=StrToBool(OptionLine);
            2014: WorkSet.Kartesianset.UseUsrDefUnits:=StrToBool(OptionLine);
            2015: WorkSet.Kartesianset.UnitValueX:=StrToFloat(OptionLine);
            2016: WorkSet.Kartesianset.UnitValueY:=StrToFloat(OptionLine);
            2017: WorkSet.Kartesianset.UnitSignX:=OptionLine;
            2018: WorkSet.Kartesianset.UnitSignY:=OptionLine;
            2019: WorkSet.Kartesianset.UnitNameX:=OptionLine;
            2020: WorkSet.Kartesianset.UnitNameY:=OptionLine;
            2021: WorkSet.Kartesianset.DrwUSignRul:=StrToBool(OptionLine);
            //MainSizing
            3000: WorkSet.MainSizing.DrawResX:=StrToInt(OptionLine);
            3001: WorkSet.MainSizing.DrawResY:=StrToInt(OptionLine);
            3002: WorkSet.MainSizing.MainWidth:=StrToInt(OptionLine);
            3003: WorkSet.MainSizing.MainHeight:=StrToInt(OptionLine);
            3004: WorkSet.MainSizing.Maximized:=StrToBool(OptionLine);
            //FontSet
            4000: WorkSet.FontSet.StatsFontScale:=StrToFloat(OptionLine);
            4001: WorkSet.FontSet.RulFontScale:=StrToFloat(OptionLine);
            4002: WorkSet.FontSet.AxisFontScale:=StrToFloat(OptionLine);
          end;
        end;
      end;
    end;

  function tprmsettings.LoadIniToGlobalSet(var RefSet:tGlobalSet; NameOrPath:UTF8string):boolean;
   var
     DefaultSet:TextFile;
     s:String;
    begin
      result:=false;
      refset:=get_Defaults;
      If FileExistsUTF8(NameOrPath)then begin
        try
          AssignFile(DefaultSet,UTF8ToSys(NameOrPath));
          {$i-}
          Reset(DefaultSet);
          {$i+}
          while not eof(DefaultSet)do begin
            readln(DefaultSet,s);
            if s<>'' then begin
              s:=Trim(s); //Leerzeichen an Stringenden entfernen
              try
                StrToOption(s,RefSet);
              except

              end;
            end;
          end;
          result:=true;
        finally
          CloseFile(DefaultSet);
        end;
      end;
    end;

//===========================================================================================================================================

  function tprmsettings.UndoSettings (MainSource:boolean;const OldSettings:tGlobalSet):tGlobalSet;
   var
     k:longInt;
    begin
      If MainSource=true then k:=0 else k:=1;
      If length(SettingsArr[k])>0 then begin
        result:=SettingsArr[k,length(SettingsArr[k])-1];
        setlength(SettingsArr[k],length(SettingsArr[k])-1);
      end else result:=OldSettings;
    end;

//===========================================================================================================================================

  procedure tprmsettings.BackupSettings (MainSource:boolean; const Settings:tGlobalSet);
   var
     i,SourceIndexBuf,k:longInt;
    begin
      If MainSource=true then k:=0 else k:=1;
      If length(SettingsArr[k])<20 then begin               //Wenn Fassungslimit des Arrays noch nicht erreicht, wird das Setting hinten drangehängt
        setlength(SettingsArr[k],length(SettingsArr[k])+1);
        SettingsArr[k,length(SettingsArr[k])-1]:=Settings;
      end else begin                                        //Wenn Fassungslimit erreicht, wird das erste Setting überschrieben durch das zweite und so weiter und das letzte an die letzte Stelle geschrieben
        for i:=0 to length(SettingsArr[k])-2 do begin
          SettingsArr[k,i]:=SettingsArr[k,i+1];
        end;
        SettingsArr[k,length(SettingsArr[k])-1]:=Settings;
      end;
    end;

//===========================================================================================================================================

  procedure tprmSettings.DelSettingsBackUps (MainSource:boolean);
   var
     SourceIndexBuf,k:longInt;
    begin
      If MainSource=true then k:=0 else k:=1;
      setlength(SettingsArr[k],0);
    end;


  function tprmSettings.BackUpCount(const MainSource:boolean):word;
    begin
      if MainSource then result:=length(SettingsArr[0]) else result:=length(settingsArr[1]);
    end;


  function tprmSettings.LoadLastFuncs (var LoadAim:tStrArr; loc_filename:String):boolean;
   var
     DefaultSet:TextFile;
     s:String;
    begin
      setlength(LoadAim,0);
      If FileExists(loc_filename)then begin
        try
          AssignFile(DefaultSet,loc_filename);
          {$i-}
          Reset(DefaultSet);
          {$i+}
          while not eof(DefaultSet)do begin
            readln(DefaultSet,s);
            if s<>'' then begin
              setlength(LastMainEditFuncs,length(LastMainEditFuncs)+1);
              setlength(LoadAim,length(LoadAim)+1);
              LastMainEditFuncs[length(LastMainEditFuncs)-1]:=s;
              LoadAim[length(LoadAim)-1]:=s;
            end;
          end;
        finally
          CloseFile(DefaultSet);
        end;
      end;
     end;


  function tprmSettings.SearchLast(const SearchStr:String):longInt;
    var
      i,le:integer;
     begin
       result:=-1;
       le:=length(LastMainEditFuncs);
       if le>0 then begin
         for i:=0 to le-1 do begin
           if SearchStr=LastMainEditFuncs[i] then result:=i;
         end;
       end;
     end;


  function tprmSettings.AddNewLastFunc (thisfunction:String):boolean;
    var
      i,le,found:longint;
     begin
       result:=false;
       le:=length(LastMainEditFuncs);
       If (thisFunction<>'')then begin
         found:=searchLast(thisfunction);
         If le<20 then begin
           If (le=0)or((le>0)and(found=-1))then begin
             setlength(LastMainEditFuncs,le+1);
             LastMainEditFuncs[le]:=ThisFunction;
             result:=true;
           end;
         end else begin
           if found=-1 then begin
             for i:=0 to le-2 do begin
               LastMainEditFuncs[i]:=LastMainEditFuncs[i+1];
             end;
             LastMainEditFuncs[le-1]:=ThisFunction;
             result:=true;
           end;
         end;

         If found>-1 then begin
           for i:=found to le-2 do begin
             LastMainEditFuncs[i]:=LastMainEditFuncs[i+1];
           end;
           LastMainEditFuncs[le-1]:=Thisfunction;
         end;
       end;
     end;


  function tprmSettings.SaveLastFuncs (loc_filename:String):boolean;
    var
     DefaultSet:TextFile;
     i,le:longInt;
    begin
      le:=length(LastMainEditFuncs);
      If le >0 then begin
        try
          AssignFile(DefaultSet,loc_filename);
          {$i-}
          ReWrite(DefaultSet);
          {$i+}
          for i:=0 to le-1 do begin
            writeln(DefaultSet,LastMainEditFuncs[i]);
          end;
        finally
          CloseFile(DefaultSet);
        end;
      end;
     end;


  procedure tprmSettings.refreshLastFuncs(var ref_results:tStrArr);
    var
    i,le:longint;
     begin
       le:=length(LastMainEditFuncs);
       setlength(ref_results,le);
       If le>0 then begin
         for i:=le-1 DownTo 0 do begin
           ref_results[i]:=LastMainEditFuncs[i];
         end;
       end;
     end;

end.

